\# $Id: Article.cls,v 1.14 2008/07/28 10:37:16 dmw Exp $
.
\# .----------------------------.
\# |				|
\# |  The ``Article'' Package	|
\# |  ---   -------   -------	|
\# |				|
\# `----------------------------'
.
.	|| One of the document classes.
.	|| reserved prefix: art-
.
.	|| Defines a header on first page, with authors, authors institutions,
.	|| and abstract. It has no chapters, but is similar to Report and book.
.	|| TODO:
.	||      n art-title etc should be doc-title etc,
.	||            as may be used globally.
.	||      o Tune spacing
.
.BeginDoc
.Section "The Article Class"
.Index class ! Article |B
.Index Class ! Article |B
This is one of the predefined document classes.
It's main features are a title, an abstract, one or more authors and their
institutions.
All of these are optional.
They should appear just after
.CW .BeginDocument .
.SubSection Commands
.EndDoc
.
\#  ____________________________
\# |				|
\# |    Title [ line1 ... ]	|
\# |    -----   -----      	|
\# |____________________________|
.
.	|| Each argument is on its own line, and is centred.
.	|| TODO: The spacing before and after title
.
.BeginDoc
.TP
.CW .Title
.CommandIndex Title |B
This command takes any number of arguments.
Each will be printed centred on a separate line as part of the document title.
Currently no part of the title appears anywhere else in the document, but in
future it might be available as part of a page header or footer.
.EndDoc
.
.de Title
.if r art-title .Error \\$[0] at most one .Title command
.nr art-title 0
.ce 1000
.vs \\n[art-title-space]u
.while \\n[.$] \{\
\&\s[\\n[art-title-size]u]\f[\\*[art-title-font]]\\$[1]\f[]\s[0]
.	br
.	shift
.\}
.ce 0
.vs
.Space \\*[art-postt]
.ns
..
.
\#  ____________________________________
\# |					|
\# |    Author name [ institution ]	|
\# |    ------ ----   -----------  	|
\# |____________________________________|
.
.	|| There may be several of these.
.
.BeginDoc
.TP
.CW ".Author name [ institution ]"
.CommandIndex Author |B
An author's name and institution.
There may be several authors, each given by a
.CW Author
command.
.EndDoc
.
.	|| If several authors then there should be the option of printing
.	|| them one underneath the other, or side by side. Each author can
.	|| be individually centred or left or right adjusted. This should
.	|| be done in the diversion for each, and the width saved.
.	|| Positioning should be done with .po and suitable calculations
.
.de Author
.ie !\\n[.$] .Error \\$[0] no name given
.el .art-doauth \\$[@]
..
.
.de art-doauth
.di art-auth\\n+[art-n]
.FillOff
.I "\\$[1]"
.shift
.if \\n[.$] .Space \\*[art-postauth]
.while \\n[.$] \{\
.	R "\\$[1]"
.	shift
.\}
.br
.FillOn
.di
.nr art-awid\\n[art-n] \\n[dl]
.	|| Now reformat centring.
.FillOff
.ll \\n[dl]u
.di x
.ce 100
.art-auth\\n[art-n]
.ce 0
.br
.di
.FillOn
.ll
.rn x art-auth\\n[art-n]
..
.
\#  ____________________________________________________
\# |							|
\# | EndAuthors [ left | centre | right ] [ row | col ]	|
\# | ----------   ----   ------   -----     ---   ---  	|
\# |____________________________________________________|
.
.	|| This is required to terminate the authors list, and format
.	|| it.
.
.BeginDoc
.TP
.CW ".EndAuthors [ left | centre | right ] [ row | col ]"
.CommandIndex EndAuthors |B
This marks the end of the author list; if not present, the authors will not be
displayed.
The arguments specify whether the individual authors have their entries left-,
centre- or right-adjusted, and whether the list of authors is displayed
side-by-side
.CW ( row|col ""
argument) or one under the other.
More authors followed by
.CW .EndAuthors
may follow.
The default positioning is side-by-side, each centred in its own space.
.EndDoc
.
.de EndAuthors
.	|| XXX TODO --- format the author list, vert or horiz
.	|| XXX - l, c, r for each author option; h, v positioning opt
.	|| default is c, h
.	|| Author and address/institution
.	|| First find gap between authors
.nr art-gap 0
.nr art-tmp 0 1
.while \\n+[art-tmp]<=\\n[art-n] .nr art-gap +\\n[art-awid\\n[art-tmp]]
.nr art-gap (\\n[.l]-\\n[art-gap])/(1+\\n[art-n])
.nr art-po \\n[.o]
.br
.mk art-atop
.po +\\n[art-gap]u
.nr art-tmp 0 1
.FillOff
.while \\n+[art-tmp]<=\\n[art-n] \{\
.	Position \\n[art-atop]u
.	art-auth\\n[art-tmp]
.	br
.	po +(u;\\n[art-gap]+\\n[art-awid\\n[art-tmp]])
.\}
.FillOn
.po \\n[art-po]u
.Position \\n[.h]u
.	|| The author count is reset so that we can have groups, say three
.	|| on one row, 2 on the next.
.nr art-n 0 1
..
.
\#  ____________________
\# |			|
\# |   BeginAbstract	|
\# |   -------------	|
\# |____________________|
.
.BeginDoc
.TP
.CW ".BeginAbstract"
.CommandIndex BeginAbstract |B
and
.TP
.CW .EndAbstract
.CommandIndex EndAbstract |B
together limit an abstract, which will be set in a slightly different way,
preceded by a heading \[lq]ABSTRACT\[rq].
.EndDoc
.
.de BeginAbstract
.if !\\n[art-inabstr]=0 .Error \\$[0] one abstract only per document
.nr art-inabstr 1
.Space \\*[art-preab]
.	|| Use a narrower line length; the amount depends on
.	|| font, pointsize and paper size, so this will have to be
.	|| settable in the future. For now make 5/6 of line length.
.RS \\n[TextWidth]/12u \\n[TextWidth]/12u
.ce 1
.BI ABSTRACT
.Space \\*[art-abhead-sp]
..
.
.de EndAbstract
.if !\\n[art-inabstr]=1 .Error \\$[0] no accompanying .BeginAbstract
.nr art-inabstr 2
.br
.	|| Reinstate line length
.RE
.Space \\*[art-post-abstr-sp]
.ns
..
.
\#  ____________________
\# |			|
\# |  TableOfContents	|
\# |  ---------------	|
\# |____________________|
.
.	|| This redefines the default in the Contents package.
.
.de TableOfContents
.ce 1
.B "Table of Contents"
.RS .5i .5i
.cont-print-table
.RE
..
.
\# .--------------------.
\# |			|
\# |  Internal macros	|
\# |  -------- ------	|
\# |			|
\# `--------------------'
.
\#  ____________________
\# |			|
\# | Article-preamble	|
\# | ------- --------	|
\# |____________________|
.
.	|| Called by BeginDocument. It outputs any stuff defined during the
.	|| preamble, or formats extra pages, such as a cover, or other stuff
.	|| which may be ``hand-formatted'', but which require sizes etc
.	|| to be set.
.
.de Article-preamble
.	|| Do initial spacing (before the title, if any)
.	|| For Article, this is the very first thing called, so since
.	|| we DO want space at the top, spring the (initially empty)
.	|| top of page trap (no space mode is ON, apparently, until
.	|| the first break).
.br \" Hopefully springs the (empty) page trap before the sp
'rs
'Position \\*[art-head-dist]
..
.
\#  ____________________
\# |			|
\# |  Initialisation	|
\# |  --------------	|
\# |____________________|
.
.	|| The spacing items
.ds art-head-dist 1i \" Spacing at top of first page of article
.ds art-postt 1.5v \" after title space
.ds art-postauth .25v \" Gap between author and address
.ds art-preab 2v \" Pre abstract space
.ds art-abhead-sp 1v \" Spacing after word ABSTRACT
.ds art-post-abstr-sp 1.5v \" Space after the abstract
.ds art-title-font B
.nr art-title-size 16z
.nr art-title-space \n[art-title-size]*12+5/10
.nr art-n 0 1
.nr art-inabstr 0
