\# $Id: Booklet.cls,v 1.11 2008/07/26 22:54:27 dmw Exp $
.
\# .--------------------.
\# |			|
\# | The Booklet class	|
\# | --- ------- -----	|
\# |			|
\# `--------------------'
.
.	|| A class for booklets of variable material - articles,
.	|| adverts, displays and so on.
.	|| Reserved prefix: bkl-
.
.BeginDoc
.Section "The Booklet class"
.Index class ! Booklet |B
This is a class designed for making informal booklets, such as newsletters and
handbooks.
It is used mainly with customised packages which manipulate sizes and styles to
suit.
.SubSection Commands
.EndDoc
.
.de Booklet-preamble
.BookletInit
..
.
.de BookletInit
.	|| Do nothing; can be redefined by packages
..
.
\#  ____________________________
\# |				|
\# |  TwoPartTitle left right	|
\# |  ------------ ---- -----	|
\# |____________________________|
.
.	|| The format is that the arguments are printed on the left and
.	|| right of the line, with a horizontal line between, separated by
.	|| digit-sized spaces.
.	|| If either of the parts is empty, extend the line to the edge
.
.BeginDoc
.TP
.CW ".TwoPartTitle left right"
.CommandIndex TwoPartTitle |B
This command prints a title in two parts, on the left and right respectively,
with a horizontal line joining them.
Either part may be empty (then the joining line is extended to the text
margin).
No additional spacing is inserted by the command.
.EndDoc
.
.de TwoPartTitle
.if !\\n[.$]=2 .Error \\$[0] takes two arguments
.ds bkl-left \\$[1]
.ds bkl-right \\$[2]
.if !'\\*[bkl-left]'' .ds bkl-left \\*[bkl-left]\0
.if !'\\*[bkl-right]'' .ds bkl-right \0\\*[bkl-right]
.nr bkl-tmp \\n[.l]-\w'\\*[bkl-left]\\*[bkl-right]'
.ti 0
\&\\*[bkl-left]\D'l \\n[bkl-tmp]u 0'\\*[bkl-right]
.	|| The following .br is absolutely crucial !!!
.	|| If not there, the line may appear somewhere else (???)
.	|| This caused me a big headache...
.br
..
.
\#  ____________________________
\# |				|
\# |   Wrap space run page foot	|
\# |   ---- ----- --- ---- ----	|
\# |____________________________|
.
.	|| This macro prepares to include a file, which follows with .so
.	|| It cannot source the file itself, as preprocessing (such as by tbl)
.	|| would not take place.
.
.BeginDoc
.TP
.CW ".Wrap space header pageopt footeropt"
.CommandIndex Wrap |B
This command acts as a \[lq]wrapper\[rq] for included \[lq]raw\[rq] article
files.
It is followed by a command
.CW ".so file" .
.IP
The first page of the file is decorated as follows: If the
.CW pageopt
argument is
.CW `` newpage '',
then the article will start on a new page, and the
.CW header
will be used; it is a name indicating which of a set of fancy strings is used
in the \[lq]running header\[rq]; these are described in those packages which
use them.
This argument may, however, have the special values
.TP
.CW empty
There will be no page header but an amount of space at the top of the page can
be introduced by the
.CW space
argument.
.TP
.CW blank
There is a blank page header of the standard width.
.TP
.CW -
There is an ordinary header, but the running header is not changed.
.Para left
The
.CW footeropt
can be one of
.CW footer
or
.CW nofooter ,
indicating that a page footer is required or not respectively (on the new page,
if taken).
.IP
If the space argument is non-zero, then this amount of space is inserted before
the article; usually used between articles on the same page.
.EndDoc
.
.de Wrap
.if \\n[.$]>4 .Error \\$[0] "takes at most four arguments"
.	|| Ensure previous article is finished
.br
.nr bkl-wrap-sp 0
.ds bkl-wrap-run BookletRun
.nr bkl-wrap-np 0
.nr bkl-wrap-nft 0
.if \\n[.$]>=1 .ds x \\$[1]
.ie "\\*[x]"-" .nr bkl-wrap-sp 0
.el .nr bkl-wrap-sp \\*[x]
.if \\n[.$]>=2 .ds bkl-wrap-run \\$[2]
.if \\n[.$]>=3 .if "\\$[3]"newpage" .nr bkl-wrap-np 1
.if \\n[.$]>=4 .if "\\$[4]"nofooter" .nr bkl-wrap-nft 1
.
.if \\n[bkl-wrap-np] \{\
.	while 1 \{\
.		if "\\*[bkl-wrap-run]"empty" \{\
.			ds page-opt empty
.			break
.		\}
.		if "\\*[bkl-wrap-run]"blank" \{\
.			ds page-opt blank
.			break
.		\}
.		if "\\*[bkl-wrap-run]"-" \{\
.			ds page-opt ord
.			break
.		\}
.		ds page-opt ord
.		RunningHeader \\*[bkl-wrap-run]
.		break
.	\}
.	NewPage
.\}
.ie \\n[bkl-wrap-nft] .SetFooterOpt blank 1
.el .SetFooterOpt ord
.if !(\\n[bkl-wrap-sp]=0) \{\
.	rs
.	sp \\n[bkl-wrap-sp]u
.\}
..
.
\#  ____________________________
\# |				|
\# |    Auth author [ date ]	|
\# |    ---- ------   ----  	|
\# |____________________________|
.
.de Auth
.br
.Space \\n[ParaSep]u
.while 1 \{\
.	if \\n[.$]=0 \{\
.		Warning \\$[0] "takes 1 or 2 args"
.		break
.	\}
.	if \\n[.$]=1 \{\
.		B "\\$[1]"
.		break
.	\}
.	if \\n[.$]=2 \{\
.		B "\\$[1]" ,
.		I "\\$[2]"
.		break
.	\}
.	break
.	if \\n[.$]>2 \{\
.		Warning \\$[0] "takes 1 or 2 args"
.		|+ \\$[*]
.		break
.	\}
.\}
.br
..
.
.BeginDoc
.TP
.CW ".Auth author [ date]"
.CommandIndex Auth |B
This identifies the author of an article (with optional date of article).
It is designed to go at the end of the article.
.EndDoc
.
\# .--------------------.
\# |			|
\# |   Initialisation	|
\# |   --------------	|
\# |			|
\# `--------------------'
.
.	|| Tailored packages should redefine this as the default
.	|| running header, if running headers are required.
.
.ds BookletRun Booklet Class
