\# $Id: Letter.cls,v 1.19 2008/07/26 22:48:34 dmw Exp $
.
\# .----------------------------.
\# |				|
\# |   Letter Document Class	|
\# |   ------ -------- -----	|
\# |				|
\# `----------------------------'
.
.	|| A Standard -markup document class
.	|| Reserved Prefix Letter or let-
.
.	|| A package for doing letters, with address and recipient
.	|| and an option for making a label file.
.
.	|| Requirements:
.
.	|| 1. Sender
.	|| 2. Addressee
.	|| 3. Logos and banner heading, customisable.
.	|| 4. Multiple letters starting afresh.
.	|| 5. Customised address according to user (.sy whoami to find out)
.	|| 6. Signatory.
.	|| 7. Postscript etc.
.	|| 8. Date (fixed or today).
.
.BeginDoc
.Section "The Letter Class"
.Index class ! Letter |B
.Index Class ! Letter |B
This class is designed to simplify customising a letter style, with its often
rather peculiar layout.
It is probably best used with a package which pre-customises many of the
individual style settings.
There is support for logos and banner headings, multiple letters, default
signatory, postscripts and dates.
The overall format of a letter is
.CommandIndex UsePackage
.BeginDisplay Text
\&.BeginLetter
[ .Addressee ]
[ .Sender ]
\&.Salutation
\&. . . (text of letter)
\&.Sign
[ .PostScript ]
\&.EndLetter
\&. . . (more letters)
.EndDisplay Text
.Para left
There are lots of hooks and handles for customising letters; these are
.Index Letter ! customising
not described in the user documentation, but are provided for those who wish to
make a customised letter package.
.EndDoc
.
\# .------------------------------------------------------------.
\# |								|
\# |			B A N N E R				|
\# |								|
\# `------------------------------------------------------------'
\# .------------.	.---------------.	.---------------.
\# |		|	|		|	|		|
\# | addressee	|	|    logo	|	|   sender	|
\# |		|	|		|	|   date	|
\# `------------'	`---------------'	`---------------'
.
.	|| Dear ...
.	|| text of letter
.	|| .Sign
.	|| PostScripts if any
.
.	|| The banner and logo may be integrated.
.	|| The vertical positioning of Banner + logo may be uneven.
.	|| The addressee and sender must not overlap horizontally.
.	|| Option for A5 letters, or A5 page for two-up.
.	|| Suggested usage
.	|| .BeginDocument Letter
.	|| .BeginLetter
.	|| [ .Addressee ]
.	|| [ .Sender ]
.	|| .Salutation
.	|| .Sign
.	|| [ .PostScript ]
.	|| .EndLetter
.	|| . . . (more letters)
.	|| .EndDocument
.
\#  ____________________
\# |			|
\# |  Letter preamble	|
\# |  ------ --------	|
\# |____________________|
.
.	|| "(A)	Set widths of addressee and sender, depending on page width.
.	|| "	The widths are a, b, with a gap in between. Probably a and
.	|| "	b should be equal, with a reasonable gap between.
.	|| "(B)	Set environments for formatting addresses - the
.	|| "	addressee and sender.
.
.de Letter-preamble
.nr let-add-width (\\n[TextWidth]-\\n[let-add-sep])/2
.NewEnv let-add-env
.BeginEnv let-add-env
.ll \\n[let-add-width]u
.fam \\*[let-add-fam]
.ps -2 \" XXX temp experimental
.	|| The indent for broken address lines
.in 5n \" XXX tunable
.na
.nh
.EndEnv let-add-env
.	|| Letters are not usually bound and are usually quite short,
.	|| so one-sided printing is the default.
.OneSided
.	|| An environment for fancy banner headings. The default is plain
.NewEnv let-banner-env
.	|| Now the folio strings
.ds page-hleft
.ds page-hcent "Page %
.ds page-hright
.	|| The turn-over message (page-fright) is set at .BeginLetter
.ds page-fright
.ds page-fcent
.ds page-fleft
..
.
\#  ____________________
\# |			|
\# |  Begin{End}Letter	|
\# |  ----- --- ------	|
\# |____________________|
.
.	|| BeginLetter should:
.	|| (A) Ensure that a page is sprung unless previous page is empty.
.	|| (B) Start page numbering at 1.
.	|| (C) Do the banner heading. This could be of many forms:
.	||     An amount of blank space; room for the banner on preprinted
.	||     stationery; an included picture or logo.
.	|| (D) Set the position of the foot of the banner.
.	|| (E) In future several positions, (e.g. at left, middle, right, or
.	||     finer) for banners with a complicated bottom outline,
.	||     or in a corner, etc., may have to be set.
.	|| (F) A diversion containing the sender could be set up.
.	||     I assume that multiple letters will have the same
.	||     sender and date, but that may not be true always.
.	||     See what is needed!
.	|| (G) Ensure correct page headers and footers are set up. E.g. if
.	||     a banner, then no header on first page of each letter, and
.	||     for single page letters, no footer may be required, to get
.	||     in a lot of information, e.g. a tear-off slip.
.	||
.	|| The particular banner and address styles should be customised
.	|| by individual letter packages for different sorts of people.
.	||
.	|| The top addresses are set at the compulsory ``.Salutation'' command.
.	||
.	|| Another possibility is to check that there is no text outside
.	|| Begin{End}Letter, as such will upset the layout.
.
.BeginDoc
.TP
.CW .BeginLetter
.CommandIndex BeginLetter |B
.TPx
.CW .EndLetter
.CommandIndex EndLetter |B
These delimit a single letter.
There may be any number of them in one document.
A letter is started with a banner heading at page 1.
If a particular letter requires more than one page then each except the last
has a \[lq]turn over\[rq] message at the bottom (by default).
.EndDoc
.
.de BeginLetter
.if !\\n[let-state]=0 .Warning \\$[0] Possible missing .EndLetter
.nr let-state 1
.	|| If a previous letter, note that .EndLetter does not throw a page.
.	|| Prepare for an empty heading at the start of the letter.
.ds page-opt empty
.ie (\\n[nl]=0)&(\\n[.k]=0) .pn 2
.el \{\
.	SetFooterOpt blank 1
.	pn 1
.	br
.	NewPage
.\}
.	|| Want a ``turn over'' message (for formal letters)
.	|| It cannot be set yet, but assumes that the greeting is on the
.	|| first page, and there is some text of the letter.
.	|| The following will need to be cancelled at the end of the letter.
.nr let-doturn-loc \\n[let-doturn]
.Banner
.	|| Restore page heading to normal
.ds page-opt ord
.if d let-send-glob .let-do-sender let-send-glob \\n[let-send-globwid]
\# \D'l \\n[.l]u 0'
.br
..
.
\#  ____________
\# |		|
\# | EndLetter	|
\# | ---------	|
\# |____________|
.
.	|| The turn message is left as specified, in case the signature
.	|| overflows.
.de EndLetter
.if !\\n[let-state]=3 .Warning \\$[0] Possible missing .Salutation or .Sign
.nr let-state 0
.let-do-signature
.	|| The following applies even if the signature forgotten.
.SetFooterOpt blank
..
.
\#  ____________________
\# |			|
\# |    SetDate date	|
\# |    ------- ----	|
\# |____________________|
.
.	|| The date given is appended after the sender at the top right.
.	|| If date=none, then don't set a date. If date=today, then format
.	|| todays date in the style the class wants, and use that.
.	|| The default is today.
.
.BeginDoc
.TP
.CW ".SetDate date"
.CommandIndex SetDate
This command sets the string to be placed in the date position to
.CW date .
If the argument is
.CW today
then the date used is \[lq]today\[rq] \[em] the day that the letter is
formatted.
It is in the
.CW PlainDate
.CommandIndex PlainDate
style.
If the user wants a different style, then use the
.CW Date
package.
If the argument is
.CW none
then the date is omitted.
.I Note :
this should be placed before
.CW .Sender
as the latter incorporates the current date, which is \[lq]today\[rq] by
default.
.EndDoc
.
.de SetDate
.ds let-date-string \\$[1]
..
\#  ____________________
\# |			|
\# |  Salutation name	|
\# |  ---------- ----	|
\# |____________________|
.
.	|| This is the ``Dear so and so ...''. Customisable.
.
.BeginDoc
.TP
.CW ".Salutation name"
.CommandIndex Salutation |B
This is obligatory after
.CW .BeginLetter ,
and generates the \[lq]Dear \*[dots]\[rq] at the beginning of the letter, using
the argument
.CW name .
.EndDoc
.
.de Salutation
.if !\\n[let-state]=1 .Warning \\$[0] Possible missing .BeginLetter
.nr let-state 2
.ds page-fright
.if \\n[let-doturn-loc] .ds page-fright \f[I]/over\f[]
.Position \\n[.h]u
.Space .3i \" XXX tunable - space before salutation
Dear \\$[1]
.Para \" XXX ???left
..
.
\#  ____________________________________
\# |					|
\# |  let-sign [ name [ greeting ] ] 	|
\# |  --- ----   ----   --------     	|
\# |____________________________________|
.
.	|| This could have all sorts of args, e.g. name, whether to say
.	|| sincerely or faithfully, whether to have a formal bracketed name.
.	|| It should be wrapped up in a ``.Sign'' command in the particular
.	|| letter package.
.	||
.	|| The args are:
.	||    name (to print in brackets with room for signature),
.	||        omitted if empty;
.	||    greeting:
.	||        f for faithfully,
.	||        s for sincerely (the default),
.	||        n to omit completely,
.	||        anything else just goes after ``Yours''.
.	|| Normally it will be indented by amount let-sign-indent
.	|| If an arg is 'f' then use faithfully. If an arg is 'n' then
.	|| don't put anything after Yours, otherwise the arg is a name
.	|| to go in brackets after signature space (used for formal letters)
.
.de let-sign
.ds let-bye sincerely
.ds let-name \\*[let-default-name]
.while \\n[.$] \{\
.	if "\\*[let-bye]"---" \{\
.		ds let-bye \\$[1]
.		shift
.		continue
.	\}
.	if "\\$[1]"n" \{\
.		ds let-bye
.		shift
.		continue
.	\}
.	if "\\$[1]"f" \{\
.		ds let-bye faithfully
.		shift
.		continue
.	\}
.	if "\\$[1]"s" \{\
.		ds let-bye sincerely
.		shift
.		continue
.	\}
.	if "\\$[1]"-b" \{\
.		ds let-bye ---
.		shift
.		continue
.	\}
.	ds let-name "\\$[1]
.	shift
.\}
.Space .3i \" XXX tunable - space before Yours etc.
.in +\\n[let-sign-indent]u
Yours \\*[let-bye]
.	|| Enough space for a signature.
.Space .5i \" XXX tunable
.if !"\\*[let-name]"" (\\*[let-name])
\# XXX
.in
.	|| XXX PostScript to do
.BeginDoc
The
.CW .Postscript
command is not implemented.
.EndDoc
..
.
\#  ____________
\# |		|
\# |    Sign	|
\# |    ----	|
\# |____________|
.
.	|| The default .Sign
.	|| We do not want the signature and the postscript to be
.	|| separated, but we also do not want the letter text to
.	|| be on a different page from the signature, so divert from
.	|| .Sign onwards, and let .EndLetter sort it out.
.
.BeginDoc
.TP
.CW ".Sign [n|f|s|-b \f[I]bye\f[]] [name]"
.CommandIndex Sign |B
This is obligatory after the text of the letter and before
.CW .EndLetter .
It generates the message \[lq]Yours sincerely\[rq] by default.
If one of the arguments is a single letter
.CW n ,
.CW f
or
.CW s
then the the farewell is none, \[lq]faithfully\[rq] or \[lq]sincerely\[rq]
respectively.
If an argument is
.CW -b
then the next argument
.I bye
is used as the signing-off phrase.
If a name is given, it is placed in brackets after room for a signature: use
this for formal letters.
.EndDoc
.
.de Sign
.if !\\n[let-state]=2 .Error \\$[0] Possible missing .Salutation
.nr let-state 3
.para-restore
.di let-signdiv
.let-sign \\$[@]
..
.
\#  ____________________
\# |			|
\# |    LetterExtra	|
\# |    -----------	|
\# |____________________|
.
.	|| This is for material after the signature, such as a questionnaire.
.	|| It will be formatted normally. May call a new page
\#
.BeginDoc
.TP
.CW .LetterExtra
.CommandIndex LetterExtra
If there is material after the signature, but before
.CW .EndLetter ,
then use this; it completes the signature and the user can insert any material.
A new page or extra space is not inserted, such details are up to the
requirements of the user.
.EndDoc
\#
.de LetterExtra
.let-do-signature
.SetFooterOpt ord
..
.
\#  ____________
\# |		|
\# |   Banner	|
\# |   ------	|
\# |____________|
.
.	|| The default banner is a .5 inch blank header
.	|| XXX This should not be a public command: swap the names
.	|| let-do-banner and Banner
.
.BeginDoc
.TP
.CW .Banner
.CommandIndex Banner |B
This produces the banner at the top of a letter.
The default is a .5 inch blank space.
.EndDoc
.
.de Banner
.rs
.BeginEnv let-banner-env
.let-do-banner
.EndEnv let-banner-env
.	|| Mark the banner foot, so that addresses can be positioned
.nr let-banner-foot \\n[nl]
..
.
\#  ____________________________
\# |				|
\# |    Sender name add ...	|
\# |    ------ ---- ---    	|
\# |____________________________|
.
.	|| This may be used once outside Begin{End}Letter, in which case
.	|| It will set as the sender (top right under banner) for all
.	|| letters. Otherwise this can be used for each letter and must
.	|| be specified individually. let-state can be 0 (global) or 1
.	|| (just after .BeginLetter).
.	|| Other possibilites are: if no args use previous; if one arg (no)
.	|| then omit global sender for this letter.
.
.de Sender
.if \\n[let-state]>1 .Error \\$[0] in wrong place
.	|| Check not already set
.if d let-send-glob .Error \\$[0] conflicting ``sender'' given
.	|| We shouldn't be within a paragraph or such
.if !\\n[.k]=0 .Error \\$[0] while setting other text
.	|| There should be no fancy layout in the sender, so do not need
.	|| The Diversion package.
.ie \\n[let-state]=0 .di let-send-glob
.el .di let-send-loc
.	|| Formats in own env, and closes diversion.
.	|| Also the date traditionally goes here, if required.
.ds x \\*[let-date-string]
.if "\\*[let-date-string]"none" .ds x
.if "\\*[let-date-string]"today" \{\
.	TodaysDate
.	ds x \\*[PlainDate]
.\}
.ie !"\\*[x]"" .let-format-add \\$[@] "\f[B]\\*[x]\f[]"
.el .let-format-add \\$[@]
.ie \\n[let-state]=0 .nr let-send-globwid \\n[dl]
.el .nr let-send-locwid \\n[dl]
. 	|| Now ready to go in where required. A local one goes in now
.if \\n[let-state]>0 .let-do-sender let-send-loc \\n[let-send-locwid]
..
.
.BeginDoc
.TP
.CW ".Sender name address ..."
.CommandIndex Sender
This may occur
.I either
once before the first
.CW .BeginLetter ,
in which case it will be used for all letters,
.I or
it may occur once at most within each letter.
It formats the name and address of the sender (at the top right of the letter,
under the banner), followed by the date if specified.
It may have any number or arguments, including none; each is printed on a
separate line.
.EndDoc
.
\#  ____________________
\# |			|
\# |  SenderName name	|
\# |  ---------- ----	|
\# |____________________|
.
.	|| Define the default name to go on formal letters if none
.	|| given in .Sign
.
.BeginDoc
.TP
.CW ".SenderName name"
.CommandIndex SenderName
This sets a name to be used as the name in
.CW .Sign
if none is given; use for formal letters only.
.EndDoc
.
.de SenderName
.ds let-default-name \\$[1]
..
.
\#  ____________________________
\# |				|
\# | Addressee [ name ] add ...	|
\# | ---------   ----   ---    	|
\# |____________________________|
.
.BeginDoc
.TP
.CW ".Addressee [ name ] add ..."
.CommandIndex Addressee
This typesets the name and address of the person to whom the letter is being
sent at the top left, under the banner.
It is not obligatory, but should appear after
.CW .BeginLetter
and before
.CW .Salutation .
Each argument is printed on a separate line.
.EndDoc
.
.de Addressee
.di let-add-div
.let-format-add \\$[@]
.Position \\n[let-banner-foot]u
.FillOff
.let-add-div
.FillOn
..
.
\#  ____________
\# |		|
\# |   NoTurn	|
\# |   ------	|
\# |____________|
.
.BeginDoc
.TP
.CW .NoTurn
.CommandIndex NoTurn |B
This suppresses the \[lq]turn\[rq] message at the foot of all but the last page
of a letter.
It affects all subsequent letters if it occurs outside a letter, or only that
letter if placed after
.CW .BeginLetter .
.EndDoc
.
.de NoTurn
.	|| Suppress the `turn' message either globally or locally
.	|| If state = 0 then between letters, so global.
.	|| If state = 1 then between .BeginLetter and .Salutation, so local
.	|| Treat any other case as erroneous.
.ie \\n[let-state]=0 .nr let-doturn 0
.el \{\
.	ie \\n[let-state]=1 .Warning \\$[0] misplaced
.	el .nr doturn-loc 0
.\}
..
.
\# .--------------------.
\# |			|
\# | Internal commands	|
\# | -------- --------	|
\# |			|
\# `--------------------'
.
\#  ____________________________
\# |				|
\# |   let-do-sender div wid	|
\# |   --- -- ------ --- ---	|
\# |____________________________|
.
.	|| Prints a pre-stored sender in a diversion.
.	|| Takes the name of the diversion and its height.
.
.de let-do-sender
.	Position \\n[let-banner-foot]u
.	po +(u;\\n[.l]-\\$[2])
.	FillOff
.	\\$[1]
.	br
.	po
.	FillOn
.\}
..
.
\#  ____________________________
\# |				|
\# |   let-format-add arg ...	|
\# |   --- ------ --- ---    	|
\# |____________________________|
.
.	|| Want to print each line of the address (one argument)
.	|| on a line of its own, not wider than the set width,
.	|| but break with an indentation if it is too long.
.	|| There is a problem with the interaction of .na, .in, .ti and .br
.	|| in giving a surious blank line, which I cannot as yet explain.
.	|| The explanation was missing dot in front of \} --- !!!
.	|| If the bug is not where you're looking, it's somewhere else ...
.
.de let-format-add
.BeginEnv let-add-env
.br
.while \\n[.$] \{\
.	ti 0
\&\\$[1]
.	br
.	shift
.\}
.br
.di
.EndEnv let-add-env
..
.
\#  ____________________
\# |			|
\# |   let-do-banner	|
\# |   --- -- ------	|
\# |____________________|
.
.	|| The default banner heading (.5 inch space)
.de let-do-banner
.Space .5i \" XXX adjustable
.	|| XXX Put in a line temporarily to see positioning
\D'l \\n[.l]u 0'
.br
..
.
\#  ____________________
\# |			|
\# |  let-do-signature	|
\# |  --- -- ---------	|
\# |____________________|
.
.	|| Does the signature. Called by EndLetter etc.
.
.de let-do-signature
.	|| A postscript may have emptied the signature
.if '\\n[.z]'let-signdiv' \{\
.	br
.	di
.	|| If the diversion length is too long for the page, output with
.	|| ordinary footer, else with empty or blank footer.
.	nr let-tmp \\n[size-paper-height]-(\\n[dn]+\\n[nl]+\\n[let-tol])
.	|| .debug EndLetter dn size-paper-height nl let-tmp let-tol
.	ie \\n[let-tmp]>0 .SetFooterOpt blank 1
.	el .Warning \\$[0] Signature may be on a different page from text.
.	FillOff
.	let-signdiv
.	FillOn
.	rm let-signdiv
.	|| In case we are on the last letter, and no .BeginLetter
.\}
..
.
\# .--------------------.
\# |			|
\# |   Initialisation	|
\# |   --------------	|
\# |			|
\# `--------------------'
.
.nr let-state 0
.	|| I think a sans-serif font is rather nice for address headings.
.	|| The standard (Helvetica) is horrible, so use LucidaSans; this
.	|| should be customisable.
.ds let-add-fam LS
.ds let-date-string today
.ds let-default-name
.nr let-doturn 1
.nr let-doturn-loc 1
.	|| Delay the following settings until pointsize worked out
.am SetSizes
.nr let-tol 1v
.nr let-add-sep 2m
.nr let-sign-indent 4m
..
