# $Id: ngrog.awk,v 1.5 2012/09/12 14:03:11 dmw Exp $

BEGIN	\
	{
		Init()
	}

END	\
	{
		Finish()
	}

# Uniquely identifying patterns
# -----------------------------

# (A) Preprocessors
# -----------------
/^\.EQ\>/	{ eqn = true; SetSo() }
/^\.G1\>/	{ pic = grap = true; SetSo() }
/^\.GS\>/	{ grn = true; SetSo() }
/^\.PS([ 0-9.<].*)?$/	\
		{ pic = true; SetSo() }
/^\.R1\>/	{ refer = true; SetSo() }
/^\.SO_END$/	{ so = false }
/^\.SO_START$/	{ so = true; SetSo1() }
/^\.TE\>/	{ in_tbl = false }
/^\.\[/		{ refer_start = true; SetSo() }
/^\.\]/		{ refer_end = true; SetSo() }
/^\.cstart$/	{ chem = true }
/^\.TS\>/	{ tbl = true; in_tbl = true; SetSo() }

# (B) Macro packages
# -------------------

# There are several cases:
# (i) The top level file using such a package must have the appropriate pattern.
#     Due to the unstructured nature of many packages this is rare.

/^\.DocumentClass\>/	\
		{ markup = true }
/^\.(Tp|Dp|De|Cx|Cl)\>/	\
		{ mdoc_old = true }

# (ii) A pattern (if found) is unique to the package.  If not found, the
#      package is still not determined.

/^\.(PH|SA)\>/	{ mm = true }
/^\.(PRINTSTYLE|START)\>/	\
		{ mom = true }
/^\.([pnil]p|sh)\>/	\
		{ me = true }

# Patterns which occur in several packages
# ----------------------------------------
/^\.Dd\>/	{ has_Dd = true }	# -mdoc -mdoc-old
/^\.(O[oc]|.* O[oc])\>/	\
		{ TryOo() }
/^\.NH\>/	{ has_NH = true }
/^\.P\>/	{ has_P = true }	# -man, -mm
/^\.PP\>/	{ has_PP = true }	# -ms, -man, -mom
/^\.SH\>/	{ has_SH = true }
/^\.TH\>/	{ if( ! in_tbl ) has_TH = true }
/^\.TL\>/	{ has_TL = true }	# -ms, -mm
/^\.[IL]P\>/	{ has_ILP = true }	# -man, -ms, -markup

function Init(){
	# Set on the command line:
	#    ThisProg
	#    Options
	#    Package
	#    InFiles

	false = 0; true = !false

	# Preprocessor flags
	chem = eqn = grap = grn = in_tbl = pic = \
	refer = refer_start = so = soelim = tbl = false

	# Macro package flags
	markup = mdoc_old = me = mm = mom = false

	# Individual  macro flags
	has_ILP = has_NH = has_Oo = has_P = has_PP = has_SH = has_TH = \
	has_TL = has_Dd = false

	First = true
}

function TryOo(){
	sub(/\\\".*/, "")
	gsub(/\"[^\"]*\"/, "")
	sub(/\".*/, "")
	sub(/^\.Oo/, " Oo ")
	sub(/^\.Oc/, " Oc ")
	sub(/ Oo$/, " Oo ")
	sub(/ Oc$/, " Oc ")
	while (/ Oo /) {
		sub(/ Oo /, " ")
		has_Oo++;
	}
	while (/ Oc /) {
		sub(/ Oc /, " ")
		has_Oo--
	}
}

function SetSo(){
	if( so ) soelim = true
}

function SetSo1(){
	if( eqn || pic || grn || tbl || chem || grap || refer ||
		( refer_start && ! refer_end) ) soelim = true
}

function Finish(){
	if( chem ){
		if( soelim ){
			printf "soelim %s | chem | ", InFiles
			soelim = false
		} else {
			printf "chem %s | ", InFiles
		}
		pic = true;    
		InFiles = ""
	}
	refer = refer || (refer_start && refer_end)

	# Note: -s flag is generated only if required before other preprocessor
	# or if preprocessor input has a .so
	printf "groff"
	PreProc(soelim, "s")
	PreProc(refer, "R")
	PreProc(grn, "g")
	PreProc(grap, "G")
	PreProc(pic, "p")
	PreProc(tbl, "t")
	PreProc(eqn, "e")

	PackFound = false
	GuessedPack = ""

	# At most one package is selected:
	if( markup ){
		DoPackage("markup");
	} else if( me ){
		DoPackage("me")
	} else if( mom ){
		DoPackage("mom")
	} else if( has_SH && has_TH ){
		DoPackage("man")
	} else if( mm || has_P ){
		DoPackage("mm")
	} else if( has_NH || has_TL || has_ILP || has_PP ){
		# .TL also occurs in -mm, .IP and .LP also occur in -man
		# .IP occurs in -markup
		# .PP also occurs in -mom and -man
		DoPackage("ms")
	} else if( has_Dd ){
		if( mdoc_old || has_Oo ){
			DoPackage("mdoc-old")
		} else {
			DoPackage("mdoc")
		}
	}
	if( ! PackFound ) {
		if( Package != "" ) printf " %s", Package
	}
	if( Options != "" ) printf " %s", Options
	if( InFiles != "" ) printf " %s", InFiles
	print ""
	if( PackFound ) {
		if (Package != "" && GuessedPack != Package) \
			Error("guessed package conflicts with given (%s)",
				Package)
	}
}

function PreProc(pre, code){
	if( pre ){
		if( First ) printf " -"
		First = false
		printf code
	}
}

function DoPackage(s){
	PackFound = true
	GuessedPack = "-" s
	printf " %s", GuessedPack
}

function Error(fmt, arg){
	printf "%s: error: ", ThisProg > "/dev/stderr"
	printf fmt, arg > "/dev/stderr"
	print "" > "/dev/stderr"
	close("/dev/stderr")
	exit 1
}
